@@ -98,7 +98,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: nmap.cc 2968 2005-12-06 22:26:00Z fyodor $ */
+/* $Id: nmap.cc 3084 2006-01-25 07:49:16Z fyodor $ */
 
 #include "nmap.h"
 #include "osscan.h"
@@ -107,6 +107,7 @@
 #include "timing.h"
 #include "NmapOps.h"
 #include "MACLookup.h"
+#include "tty.h"
 #ifdef WIN32
 #include "winfix.h"
 #endif
@@ -198,21 +199,19 @@ static int parse_bounce_argument(struct ftpinfo *ftp, char *url) {
 int nmap_main(int argc, char *argv[]) {
   char *p, *q;
   int i, arg;
+  long l;
   unsigned int targetno;
   size_t j, argvlen;
   FILE *inputfd = NULL, *excludefd = NULL;
   char *host_spec = NULL, *exclude_spec = NULL;
-  short fastscan=0, randomize=1, resolve_all=0;
+  short fastscan=0, randomize=1;
   short quashargv = 0;
-  int numhosts_scanned = 0;
   char **host_exp_group;
   char *idleProxy = NULL; /* The idle host used to "Proxy" an Idlescan */
   int num_host_exp_groups = 0;
   char *machinefilename = NULL, *kiddiefilename = NULL, 
     *normalfilename = NULL, *xmlfilename = NULL;
   HostGroupState *hstate = NULL;
-  int numhosts_up = 0;
-  int starttime;
   char *endptr = NULL;
   struct scan_lists *ports = NULL;
   TargetGroup *exclude_group = NULL;
@@ -228,6 +227,7 @@ int nmap_main(int argc, char *argv[]) {
   char **fakeargv;
   Target *currenths;
   vector<Target *> Targets;
+  char *portlist = NULL; /* Ports list specified by user */
   char *proberr;
   char emptystring[1];
   int sourceaddrwarning = 0; /* Have we warned them yet about unguessable
@@ -250,20 +250,32 @@ int nmap_main(int argc, char *argv[]) {
       {"help", no_argument, 0, 'h'},
       {"iflist", no_argument, 0, 0},
       {"max_parallelism", required_argument, 0, 'M'},
+      {"max-parallelism", required_argument, 0, 'M'},
       {"min_parallelism", required_argument, 0, 0},
+      {"min-parallelism", required_argument, 0, 0},
       {"timing", required_argument, 0, 'T'},
       {"timing", no_argument, 0, 0},
       {"max_rtt_timeout", required_argument, 0, 0},
+      {"max-rtt-timeout", required_argument, 0, 0},
       {"min_rtt_timeout", required_argument, 0, 0},
+      {"min-rtt-timeout", required_argument, 0, 0},
       {"initial_rtt_timeout", required_argument, 0, 0},
+      {"initial-rtt-timeout", required_argument, 0, 0},
       {"excludefile", required_argument, 0, 0},
       {"exclude", required_argument, 0, 0},
       {"max_hostgroup", required_argument, 0, 0},
+      {"max-hostgroup", required_argument, 0, 0},
       {"min_hostgroup", required_argument, 0, 0},
+      {"min-hostgroup", required_argument, 0, 0},
       {"scanflags", required_argument, 0, 0},
       {"host_timeout", required_argument, 0, 0},
+      {"host-timeout", required_argument, 0, 0},
       {"scan_delay", required_argument, 0, 0},
+      {"scan-delay", required_argument, 0, 0},
       {"max_scan_delay", required_argument, 0, 0},
+      {"max-scan-delay", required_argument, 0, 0},
+      {"max_retries", required_argument, 0, 0},
+      {"max-retries", required_argument, 0, 0},
       {"oA", required_argument, 0, 0},  
       {"oN", required_argument, 0, 0},
       {"oM", required_argument, 0, 0},  
@@ -275,17 +287,27 @@ int nmap_main(int argc, char *argv[]) {
       {"iR", required_argument, 0, 0},
       {"sI", required_argument, 0, 0},  
       {"source_port", required_argument, 0, 'g'},
+      {"source-port", required_argument, 0, 'g'},
       {"randomize_hosts", no_argument, 0, 0},
+      {"randomize-hosts", no_argument, 0, 0},
       {"osscan_limit", no_argument, 0, 0}, /* skip OSScan if no open ports */
+      {"osscan-limit", no_argument, 0, 0}, /* skip OSScan if no open ports */
       {"osscan_guess", no_argument, 0, 0}, /* More guessing flexability */
+      {"osscan-guess", no_argument, 0, 0}, /* More guessing flexability */
       {"fuzzy", no_argument, 0, 0}, /* Alias for osscan_guess */
       {"packet_trace", no_argument, 0, 0}, /* Display all packets sent/rcv */
+      {"packet-trace", no_argument, 0, 0}, /* Display all packets sent/rcv */
       {"version_trace", no_argument, 0, 0}, /* Display -sV related activity */
+      {"version-trace", no_argument, 0, 0}, /* Display -sV related activity */
       {"data_length", required_argument, 0, 0},
+      {"data-length", required_argument, 0, 0},
       {"send_eth", no_argument, 0, 0},
+      {"send-eth", no_argument, 0, 0},
       {"send_ip", no_argument, 0, 0},
+      {"send-ip", no_argument, 0, 0},
       {"stylesheet", required_argument, 0, 0},
       {"no_stylesheet", no_argument, 0, 0},
+      {"no-stylesheet", no_argument, 0, 0},
       {"webxml", no_argument, 0, 0},
       {"rH", no_argument, 0, 0},
       {"vv", no_argument, 0, 0},
@@ -293,14 +315,24 @@ int nmap_main(int argc, char *argv[]) {
       {"privileged", no_argument, 0, 0},
       {"mtu", required_argument, 0, 0},
       {"append_output", no_argument, 0, 0},
+      {"append-output", no_argument, 0, 0},
       {"noninteractive", no_argument, 0, 0},
       {"spoof_mac", required_argument, 0, 0},
+      {"spoof-mac", required_argument, 0, 0},
       {"thc", no_argument, 0, 0},  
+      {"badsum", no_argument, 0, 0},  
       {"ttl", required_argument, 0, 0}, /* Time to live */
       {"allports", no_argument, 0, 0},
       {"version_intensity", required_argument, 0, 0},
+      {"version-intensity", required_argument, 0, 0},
       {"version_light", no_argument, 0, 0},
+      {"version-light", no_argument, 0, 0},
       {"version_all", no_argument, 0, 0},
+      {"version-all", no_argument, 0, 0},
+      {"system_dns", no_argument, 0, 0},
+      {"system-dns", no_argument, 0, 0},
+      {"dns_servers", required_argument, 0, 0},
+      {"dns-servers", required_argument, 0, 0},
       {0, 0, 0, 0}
     };
 
@@ -324,24 +356,24 @@ int nmap_main(int argc, char *argv[]) {
   while((arg = getopt_long_only(argc,fakeargv,"6Ab:D:d::e:Ffg:hIi:M:m:nOo:P:p:qRrS:s:T:Vv", long_options, &option_index)) != EOF) {
     switch(arg) {
     case 0:
-      if (strcmp(long_options[option_index].name, "max_rtt_timeout") == 0) {
-	o.setMaxRttTimeout(atoi(optarg));
-	if (o.maxRttTimeout() < 5) {
-	  fatal("max_rtt_timeout is given in milliseconds and must be at least 5");
-	}       
-        if (o.maxRttTimeout() < 20) {
-	  error("WARNING: You specified a round-trip time timeout (%d ms) that is EXTRAORDINARILY SMALL.  Accuracy may suffer.", o.maxRttTimeout());
+      if (optcmp(long_options[option_index].name, "max-rtt-timeout") == 0) {
+	l = tval2msecs(optarg);
+	if (l < 5) fatal("Bogus --max-rtt-timeout argument specified, must be at least 5");
+        if (l < 20) {
+	  error("WARNING: You specified a round-trip time timeout (%ld ms) that is EXTRAORDINARILY SMALL.  Accuracy may suffer.", l);
 	}
-      } else if (strcmp(long_options[option_index].name, "min_rtt_timeout") == 0) {
-	o.setMinRttTimeout(atoi(optarg));
-	if (o.minRttTimeout() > 50000) {
-	  error("Warning:  min_rtt_timeout is given in milliseconds, your value seems pretty large.");
-	}
-      } else if (strcmp(long_options[option_index].name, "initial_rtt_timeout") == 0) {
-	o.setInitialRttTimeout(atoi(optarg));
-	if (o.initialRttTimeout() <= 0) {
-	  fatal("initial_rtt_timeout must be greater than 0");
+	o.setMaxRttTimeout(l);
+      } else if (optcmp(long_options[option_index].name, "min-rtt-timeout") == 0) {
+	l = tval2msecs(optarg);
+	if (l < 0) fatal("Bogus --min-rtt-timeout argument specified");
+	if (l > 50000) {
+	  error("Warning:  min-rtt-timeout is given in milliseconds, your value seems pretty large.");
 	}
+	o.setMinRttTimeout(l);
+      } else if (optcmp(long_options[option_index].name, "initial-rtt-timeout") == 0) {
+	l = tval2msecs(optarg);
+	if (l <= 0) fatal("Bogus --initial-rtt-timeout argument specified.  Must be positive");
+	o.setInitialRttTimeout(l);
       } else if (strcmp(long_options[option_index].name, "excludefile") == 0) {
         excludefd = fopen(optarg, "r");
         if (!excludefd) {
@@ -351,12 +383,12 @@ int nmap_main(int argc, char *argv[]) {
 	if (excludefd)
 	  fatal("--excludefile and --exclude options are mutually exclusive.");
 	exclude_spec = strdup(optarg);
-      } else if (strcmp(long_options[option_index].name, "max_hostgroup") == 0) {
+      } else if (optcmp(long_options[option_index].name, "max-hostgroup") == 0) {
 	o.setMaxHostGroupSz(atoi(optarg));
-      } else if (strcmp(long_options[option_index].name, "min_hostgroup") == 0) {
+      } else if (optcmp(long_options[option_index].name, "min-hostgroup") == 0) {
 	o.setMinHostGroupSz(atoi(optarg));
 	if (atoi(optarg) > 100)
-	  error("Warning: You specified a highly aggressive --min_hostgroup.");
+	  error("Warning: You specified a highly aggressive --min-hostgroup.");
       } else if (strcmp(long_options[option_index].name, "scanflags") == 0) {
 	o.scanflags = parse_scanflags(optarg);
 	if (o.scanflags < 0) {
@@ -364,19 +396,17 @@ int nmap_main(int argc, char *argv[]) {
 	}
       } else if (strcmp(long_options[option_index].name, "iflist") == 0 ) {
 	iflist = true;
-      } else if (strcmp(long_options[option_index].name, "min_parallelism") == 0 ) {
+      } else if (optcmp(long_options[option_index].name, "min-parallelism") == 0 ) {
 	o.min_parallelism = atoi(optarg); 
-	if (o.min_parallelism < 1) fatal("Argument to --min_parallelism must be at least 1!");
+	if (o.min_parallelism < 1) fatal("Argument to --min-parallelism must be at least 1!");
 	if (o.min_parallelism > 100) {
-	  error("Warning: Your --min_parallelism option is absurdly high! Don't complain to Fyodor if all hell breaks loose!");
+	  error("Warning: Your --min-parallelism option is pretty high!  This can hurt reliability.");
 	}
-      } else if (strcmp(long_options[option_index].name, "host_timeout") == 0) {
-	o.host_timeout = strtoul(optarg, NULL, 10);
-	if (o.host_timeout <= 200) {
-	  fatal("host_timeout is given in milliseconds and must be greater than 200");
-	}
-	if (o.host_timeout <= 1000) {
-	  error("host_timeout is given in milliseconds, so you specified less than a second (%lims). This is allowed but not recommended.", o.host_timeout);
+      } else if (optcmp(long_options[option_index].name, "host-timeout") == 0) {	l = tval2msecs(optarg);
+	if (l <= 200) fatal("--host-timeout must be at least 200 milliseconds");
+	o.host_timeout = l;
+	if (o.host_timeout < 1000) {
+	  error("host-timeout is given in milliseconds, so you specified less than a second (%lims). This is allowed but not recommended.", o.host_timeout);
 	}
       } else if (strcmp(long_options[option_index].name, "ttl") == 0) {
 	o.ttl = atoi(optarg);
@@ -385,69 +415,75 @@ int nmap_main(int argc, char *argv[]) {
 	}
       } else if (strcmp(long_options[option_index].name, "datadir") == 0) {
 	o.datadir = strdup(optarg);
-      } else if (strcmp(long_options[option_index].name, "append_output") == 0) {
+      } else if (optcmp(long_options[option_index].name, "append-output") == 0) {
 	o.append_output = 1;
       } else if (strcmp(long_options[option_index].name, "noninteractive") == 0) {
-	/* Do nothing */
-      } else if (strcmp(long_options[option_index].name, "spoof_mac") == 0) {
+	o.noninteractive = true;
+      } else if (optcmp(long_options[option_index].name, "spoof-mac") == 0) {
 	/* I need to deal with this later, once I'm sure that I have output
 	   files set up, --datadir, etc. */
 	spoofmac = optarg;
       } else if (strcmp(long_options[option_index].name, "allports") == 0) {
         o.override_excludeports = 1;
-      } else if (strcmp(long_options[option_index].name, "version_intensity") == 0) {
+      } else if (optcmp(long_options[option_index].name, "version-intensity") == 0) {
 	o.version_intensity = atoi(optarg);
 	if (o.version_intensity < 0 || o.version_intensity > 9)
-		fatal("version_intensity must be between 0 and 9");
-      } else if (strcmp(long_options[option_index].name, "version_light") == 0) {
+		fatal("version-intensity must be between 0 and 9");
+      } else if (optcmp(long_options[option_index].name, "version-light") == 0) {
         o.version_intensity = 2;
-      } else if (strcmp(long_options[option_index].name, "version_all") == 0) {
+      } else if (optcmp(long_options[option_index].name, "version-all") == 0) {
         o.version_intensity = 9;
-      } else if (strcmp(long_options[option_index].name, "scan_delay") == 0) {
-	o.scan_delay = atoi(optarg);
-	if (o.scan_delay <= 0) {
-	  fatal("scan_delay must be greater than 0");
-	}
+      } else if (optcmp(long_options[option_index].name, "scan-delay") == 0) {
+	l = tval2msecs(optarg);
+	if (l < 0) fatal("Bogus --scan-delay argument specified.");
+	o.scan_delay = l;
 	if (o.scan_delay > o.maxTCPScanDelay()) o.setMaxTCPScanDelay(o.scan_delay);
 	if (o.scan_delay > o.maxUDPScanDelay()) o.setMaxUDPScanDelay(o.scan_delay);
 	o.max_parallelism = 1;
-      } else if (strcmp(long_options[option_index].name, "max_scan_delay") == 0) {
-	unsigned int scand = atoi(optarg);
-	if (scand < 0) {
-	  fatal("max_scan_delay must be greater than 0");
-	}
-	o.setMaxTCPScanDelay(scand);
-	o.setMaxUDPScanDelay(scand);
-      } else if (strcmp(long_options[option_index].name, "randomize_hosts") == 0
+      } else if (optcmp(long_options[option_index].name, "max-scan-delay") == 0) {
+	l = tval2msecs(optarg);
+	if (l < 0) fatal("--max-scan-delay cannot be negative.");
+	o.setMaxTCPScanDelay(l);
+	o.setMaxUDPScanDelay(l);
+      } else if (optcmp(long_options[option_index].name, "max-retries") == 0) {
+        int num_retrans = atoi(optarg);
+        if (num_retrans < 0)
+          fatal("max-retransmissions must be positive");
+        o.setMaxRetransmissions(num_retrans);
+      } else if (optcmp(long_options[option_index].name, "randomize-hosts") == 0
 		 || strcmp(long_options[option_index].name, "rH") == 0) {
 	o.randomize_hosts = 1;
 	o.ping_group_sz = PING_GROUP_SZ * 4;
-      } else if (strcmp(long_options[option_index].name, "osscan_limit")  == 0) {
+      } else if (optcmp(long_options[option_index].name, "osscan-limit")  == 0) {
 	o.osscan_limit = 1;
-      } else if (strcmp(long_options[option_index].name, "osscan_guess")  == 0
+      } else if (optcmp(long_options[option_index].name, "osscan-guess")  == 0
                  || strcmp(long_options[option_index].name, "fuzzy") == 0) {
 	o.osscan_guess = 1;
-      } else if (strcmp(long_options[option_index].name, "packet_trace") == 0) {
+      } else if (optcmp(long_options[option_index].name, "packet-trace") == 0) {
 	o.setPacketTrace(true);
-      } else if (strcmp(long_options[option_index].name, "version_trace") == 0) {
+      } else if (optcmp(long_options[option_index].name, "version-trace") == 0) {
 	o.setVersionTrace(true);
 	o.debugging++;
-      } else if (strcmp(long_options[option_index].name, "data_length") == 0) {
+      } else if (optcmp(long_options[option_index].name, "data-length") == 0) {
 	o.extra_payload_length = atoi(optarg);
 	if (o.extra_payload_length < 0) {
-	  fatal("data_length must be greater than 0");
+	  fatal("data-length must be greater than 0");
 	} else if (o.extra_payload_length > 0) {
 	  o.extra_payload = (char *) safe_malloc(o.extra_payload_length);
 	  get_random_bytes(o.extra_payload, o.extra_payload_length);
 	}
-      } else if (strcmp(long_options[option_index].name, "send_eth") == 0) {
+      } else if (optcmp(long_options[option_index].name, "send-eth") == 0) {
 	o.sendpref = PACKET_SEND_ETH_STRONG;
-      } else if (strcmp(long_options[option_index].name, "send_ip") == 0) {
+      } else if (optcmp(long_options[option_index].name, "send-ip") == 0) {
 	o.sendpref = PACKET_SEND_IP_STRONG;
       } else if (strcmp(long_options[option_index].name, "stylesheet") == 0) {
 	o.setXSLStyleSheet(optarg);
-      } else if (strcmp(long_options[option_index].name, "no_stylesheet") == 0) {
+      } else if (optcmp(long_options[option_index].name, "no-stylesheet") == 0) {
 	o.setXSLStyleSheet(NULL);
+      } else if (strcmp(long_options[option_index].name, "system-dns") == 0) {
+        o.mass_dns = false;
+      } else if (optcmp(long_options[option_index].name, "dns-servers") == 0) {
+        o.dns_servers = strdup(optarg);
       } else if (strcmp(long_options[option_index].name, "webxml") == 0) {
 	o.setXSLStyleSheet("http://www.insecure.org/nmap/data/nmap.xsl");
       } else if (strcmp(long_options[option_index].name, "oN") == 0) {
@@ -472,6 +508,8 @@ int nmap_main(int argc, char *argv[]) {
       } else if (strcmp(long_options[option_index].name, "thc") == 0) {
 	printf("!!Greets to Van Hauser, Plasmoid, Skyper and the rest of THC!!\n");
 	exit(0);
+      } else if (strcmp(long_options[option_index].name, "badsum") == 0) {
+	o.badsum = 1;
       }
       else if (strcmp(long_options[option_index].name, "iL") == 0) {
 	if (inputfd) {
@@ -542,7 +580,7 @@ int nmap_main(int argc, char *argv[]) {
 	  if (resolve(p, &o.decoys[o.numdecoys])) {
 	    o.numdecoys++;
 	  } else {
-	    fatal("Failed to resolve decoy host: %s (must be hostname or IP address)", optarg);
+	    fatal("Failed to resolve decoy host: %s (must be hostname or IP address)", p);
 	  }
 	}
 	if (q) {
@@ -565,7 +603,7 @@ int nmap_main(int argc, char *argv[]) {
     case 'g': 
       o.magic_port = atoi(optarg);
       o.magic_port_set = 1;
-      if (!o.magic_port) fatal("-g needs nonzero argument");
+      if (o.magic_port == 0) error("WARNING: a source port of zero may not work on all systems.");
       break;    
     case 'h': printusage(argv[0], 0); break;
     case '?': printusage(argv[0], -1); break;
@@ -590,7 +628,7 @@ int nmap_main(int argc, char *argv[]) {
       o.max_parallelism = atoi(optarg); 
       if (o.max_parallelism < 1) fatal("Argument to -M must be at least 1!");
       if (o.max_parallelism > 900) {
-	error("Warning: Your max_parallelism (-M) option is absurdly high! Don't complain to Fyodor if all hell breaks loose!");
+	error("Warning: Your max-parallelism (-M) option is extraordinarily high, which can hurt reliability");
       }
       break;
     case 'm': 
@@ -675,14 +713,12 @@ int nmap_main(int argc, char *argv[]) {
       }
       break;
     case 'p': 
-      if (ports)
+      if (ports || portlist)
 	fatal("Only 1 -p option allowed, separate multiple ranges with commas.");
-      ports = getpts(optarg);
-      if (!ports)
-	fatal("Your port specification string is not parseable");
+      portlist = strdup(optarg);
       break;
     case 'q': quashargv++; break;
-    case 'R': resolve_all++; break;
+    case 'R': o.resolve_all++; break;
     case 'r': 
       randomize = 0;
       break;
@@ -748,6 +784,7 @@ int nmap_main(int argc, char *argv[]) {
 	o.setMaxRttTimeout(1250);
 	o.setInitialRttTimeout(500);
         o.setMaxTCPScanDelay(10);
+        o.setMaxRetransmissions(6);
       } else if (*optarg == '5' || (strcasecmp(optarg, "Insane") == 0)) {
 	o.timing_level = 5;
 	o.setMinRttTimeout(50);
@@ -755,8 +792,9 @@ int nmap_main(int argc, char *argv[]) {
 	o.setInitialRttTimeout(250);
 	o.host_timeout = 900000;
         o.setMaxTCPScanDelay(5);
+        o.setMaxRetransmissions(2);
       } else {
-	fatal("Unknown timing mode (-T argment).  Use either \"Paranoid\", \"Sneaky\", \"Polite\", \"Normal\", \"Aggressive\", \"Insane\" or a number from 0 (Paranoid) to 5 (Insane)");
+	fatal("Unknown timing mode (-T argument).  Use either \"Paranoid\", \"Sneaky\", \"Polite\", \"Normal\", \"Aggressive\", \"Insane\" or a number from 0 (Paranoid) to 5 (Insane)");
       }
       break;
     case 'V': 
@@ -771,6 +809,8 @@ int nmap_main(int argc, char *argv[]) {
     win_init();
 #endif
 
+  tty_init(); // Put the keyboard in raw mode
+
 #if HAVE_SIGNAL
   if (!o.debugging)
     signal(SIGSEGV, sigdie); 
@@ -815,6 +855,14 @@ int nmap_main(int argc, char *argv[]) {
     fatal("The fast scan (-F) is incompatible with ping scan");
   }
 
+  if (portlist) {
+    ports = getpts(portlist);
+    if (!ports)
+      fatal("Your port specification string is not parseable");
+    free(portlist);
+    portlist = NULL;
+  }
+
   if (fastscan && ports) {
     fatal("You can specify fast scan (-F) or explicitly select individual ports (-p), but not both");
   } else if (fastscan && o.ipprotscan) {
@@ -829,7 +877,7 @@ int nmap_main(int argc, char *argv[]) {
 
 #ifdef WIN32
   if (o.sendpref & PACKET_SEND_IP) {
-	  error("WARNING: raw IP (rather than raw ethernet) packet sending attempted on Windows. This probably won't work.  Consider --send_eth next time.\n");
+	  error("WARNING: raw IP (rather than raw ethernet) packet sending attempted on Windows. This probably won't work.  Consider --send-eth next time.\n");
 
   }
 #endif
@@ -848,7 +896,7 @@ int nmap_main(int argc, char *argv[]) {
       while(*p) { 
 	if (*p == ':') p++;
 	if (isxdigit(*p) && isxdigit(*(p+1))) {
-	  if (pos >= 6) fatal("Bogus --spoof_mac value encountered (%s) -- only up to 6 bytes permitted", spoofmac);
+	  if (pos >= 6) fatal("Bogus --spoof-mac value encountered (%s) -- only up to 6 bytes permitted", spoofmac);
 	  tmphex[0] = *p; tmphex[1] = *(p+1); tmphex[2] = '\0';
 	  mac_data[pos] = (u8) strtol(tmphex, NULL, 16);
 	  pos++;
@@ -858,7 +906,7 @@ int nmap_main(int argc, char *argv[]) {
       if (*p) {
 	/* Failed to parse it as a MAC prefix -- treating as a vendor substring instead */
 	if (!MACCorp2Prefix(spoofmac, mac_data))
-	  fatal("Could not parse as a prefix nor find as a vendor substring the given --spoof_mac argument: %s.  If you are giving hex digits, there must be an even number of them.", spoofmac);
+	  fatal("Could not parse as a prefix nor find as a vendor substring the given --spoof-mac argument: %s.  If you are giving hex digits, there must be an even number of them.", spoofmac);
 	pos = 3;
       }
     }
@@ -1030,8 +1078,6 @@ int nmap_main(int argc, char *argv[]) {
       shortfry(ports->prots, ports->prot_count); 
   }
   
-  starttime = time(NULL);
-  
   /* Time to create a hostgroup state object filled with all the requested
      machines */
   host_exp_group = (char **) safe_malloc(o.ping_group_sz * sizeof(char *));
@@ -1053,7 +1099,7 @@ int nmap_main(int argc, char *argv[]) {
 	(host_spec = grab_next_host_spec(inputfd, argc, fakeargv))) {
     host_exp_group[num_host_exp_groups++] = strdup(host_spec);
     // For purposes of random scan
-    if (o.max_ips_to_scan && o.max_ips_to_scan <= numhosts_scanned + num_host_exp_groups)
+    if (o.max_ips_to_scan && o.max_ips_to_scan <= o.numhosts_scanned + num_host_exp_groups)
       break;
   }
 
@@ -1063,7 +1109,7 @@ int nmap_main(int argc, char *argv[]) {
 			      host_exp_group, num_host_exp_groups);
 
   do {
-    ideal_scan_group_sz = determineScanGroupSize(numhosts_scanned, ports);
+    ideal_scan_group_sz = determineScanGroupSize(o.numhosts_scanned, ports);
     while(Targets.size() < ideal_scan_group_sz) {
       currenths = nexthost(hstate, exclude_group, ports, &(o.pingtype));
       if (!currenths) {
@@ -1074,7 +1120,7 @@ int nmap_main(int argc, char *argv[]) {
 	num_host_exp_groups = 0;
 	/* Now grab any new expressions */
 	while(num_host_exp_groups < o.ping_group_sz && 
-	      (!o.max_ips_to_scan ||  o.max_ips_to_scan > numhosts_scanned + num_host_exp_groups) &&
+	      (!o.max_ips_to_scan ||  o.max_ips_to_scan > o.numhosts_scanned + num_host_exp_groups) &&
 	      (host_spec = grab_next_host_spec(inputfd, argc, fakeargv))) {
 	  // For purposes of random scan
 	  host_exp_group[num_host_exp_groups++] = strdup(host_spec);
@@ -1090,25 +1136,15 @@ int nmap_main(int argc, char *argv[]) {
 	if (!currenths)
 	  break;
       }
-      numhosts_scanned++;
+      o.numhosts_scanned++;
     
       if (currenths->flags & HOST_UP && !o.listscan) 
-	numhosts_up++;
-    
-      /* Lookup the IP */
-      if (((currenths->flags & HOST_UP) || resolve_all) && !o.noresolve) {
-	if (currenths->TargetSockAddr(&ss, &sslen) != 0)
-	  fatal("Failed to get target socket address.");
-	if (getnameinfo((struct sockaddr *)&ss, sslen, hostname, 
-			sizeof(hostname), NULL, 0, NI_NAMEREQD) == 0) {
-	  currenths->setHostName(hostname);
-	}
-      }
+	o.numhosts_up++;
     
       if (o.pingscan || o.listscan) {
 	/* We're done with the hosts */
 	log_write(LOG_XML, "<host>");
-	write_host_status(currenths, resolve_all);
+	write_host_status(currenths, o.resolve_all);
 	printmacinfo(currenths);
 	//	if (currenths->flags & HOST_UP)
 	//  log_write(LOG_NORMAL|LOG_SKID|LOG_STDOUT,"\n");
@@ -1158,7 +1194,7 @@ int nmap_main(int argc, char *argv[]) {
 	if (Targets.size() > 0 && 
 	    strcmp(Targets[Targets.size() - 1]->deviceName(), currenths->deviceName())) {
 	  returnhost(hstate);
-	  numhosts_scanned--; numhosts_up--;
+	  o.numhosts_scanned--; o.numhosts_up--;
 	  break;
 	}
 	o.decoys[o.decoyturn] = currenths->v4source();
@@ -1169,12 +1205,17 @@ int nmap_main(int argc, char *argv[]) {
     if (Targets.size() == 0)
       break; /* Couldn't find any more targets */
     
+    // Set the variable for status printing
+    o.numhosts_scanning = Targets.size();
+    
     // Our source must be set in decoy list because nexthost() call can
     // change it (that issue really should be fixed when possible)
     if (o.af() == AF_INET && o.RawScan())
       o.decoys[o.decoyturn] = Targets[0]->v4source();
     
     /* I now have the group for scanning in the Targets vector */
+
+    // Ultra_scan sets o.scantype for us so we don't have to worry
     if (o.synscan)
       ultra_scan(Targets, ports, SYN_SCAN);
     
@@ -1208,17 +1249,26 @@ int nmap_main(int argc, char *argv[]) {
     /* These lame functions can only handle one target at a time */
     for(targetno = 0; targetno < Targets.size(); targetno++) {
       currenths = Targets[targetno];
-      if (o.idlescan) idle_scan(currenths, ports->tcp_ports, 
+      if (o.idlescan) {
+         o.scantype = IDLE_SCAN;
+         keyWasPressed(); // Check if a status message should be printed
+         idle_scan(currenths, ports->tcp_ports, 
 				ports->tcp_count, idleProxy);
+      }
       if (o.bouncescan) {
+         o.scantype = BOUNCE_SCAN;
+         keyWasPressed(); // Check if a status message should be printed
 	if (ftp.sd <= 0) ftp_anon_connect(&ftp);
 	if (ftp.sd > 0) bounce_scan(currenths, ports->tcp_ports, 
 				    ports->tcp_count, &ftp);
       }
     }
 
-    if (o.servicescan)
+    if (o.servicescan) {
+      o.scantype = SERVICE_SCAN; 
+      keyWasPressed(); // Check if a status message should be printed
       service_scan(Targets);
+    }
 
     for(targetno = 0; targetno < Targets.size(); targetno++) {
       currenths = Targets[targetno];
@@ -1230,12 +1280,13 @@ int nmap_main(int argc, char *argv[]) {
       if (o.servicescan || o.rpcscan)  pos_scan(currenths, NULL, 0, RPC_SCAN);
 
       // Should be host parallelized.  Though rarely takes a huge amt. of time.
-      if (o.osscan) 
+      if (o.osscan) {
 	os_scan(currenths);
+      }
 
     /* Now I can do the output and such for each host */
       log_write(LOG_XML, "<host>");
-      write_host_status(currenths, resolve_all);
+      write_host_status(currenths, o.resolve_all);
       if (currenths->timedOut(NULL)) {
 	log_write(LOG_NORMAL|LOG_SKID|LOG_STDOUT,"Skipping host %s due to host timeout\n", 
 		  currenths->NameIP(hostname, sizeof(hostname)));
@@ -1262,7 +1313,7 @@ int nmap_main(int argc, char *argv[]) {
       delete currenths;
       Targets.pop_back();
     }
-  } while(!o.max_ips_to_scan || o.max_ips_to_scan > numhosts_scanned);
+  } while(!o.max_ips_to_scan || o.max_ips_to_scan > o.numhosts_scanned);
   
   delete hstate;
   if (exclude_group)
@@ -1276,7 +1327,7 @@ int nmap_main(int argc, char *argv[]) {
   num_host_exp_groups = 0;
   free(host_exp_group);
 
-  printfinaloutput(numhosts_scanned, numhosts_up, starttime);
+  printfinaloutput();
 
   if (ports) {
     free(ports->tcp_ports);
@@ -1322,8 +1373,9 @@ int gather_logfile_resumption_state(char *fname, int *myargc, char ***myargv) {
   filestr[filelen - 1] = '\0';
 
   /* First goal is to find the nmap args */
-  p = strstr(filestr, " as: ");
-  p += 5;
+  if ((p = strstr(filestr, " as: ")))
+    p += 5;
+  else fatal("Unable to parse supposed log file %s.  Are you sure this is an Nmap output file?", fname);
   while(*p && !isspace((int) *p))
     p++;
   if (!*p) fatal("Unable to parse supposed log file %s.  Sorry", fname);
@@ -1335,13 +1387,13 @@ int gather_logfile_resumption_state(char *fname, int *myargc, char ***myargv) {
     fatal("Unable to parse supposed log file %s.  Perhaps the Nmap execution had not finished at least one host?  In that case there is no use \"resuming\"", fname);
 
 
-  strcpy(nmap_arg_buffer, "nmap --append_output ");
+  strcpy(nmap_arg_buffer, "nmap --append-output ");
   if ((q-p) + 21 + 1 >= (int) sizeof(nmap_arg_buffer)) fatal("0verfl0w");
   memcpy(nmap_arg_buffer + 21, p, q-p);
   nmap_arg_buffer[21 + q-p] = '\0';
 
-  if (strstr(nmap_arg_buffer, "--randomize_hosts") != NULL) {
-    error("WARNING:  You are attempting to resume a scan which used --randomize_hosts.  Some hosts in the last randomized batch make be missed and others may be repeated once");
+  if (strstr(nmap_arg_buffer, "--randomize-hosts") != NULL) {
+    error("WARNING:  You are attempting to resume a scan which used --randomize-hosts.  Some hosts in the last randomized batch make be missed and others may be repeated once");
   }
 
   *myargc = arg_parse(nmap_arg_buffer, myargv);
@@ -1446,7 +1498,14 @@ struct scan_lists *getpts(char *origexpr) {
   int i;
   int tcpportcount = 0, udpportcount = 0, protcount = 0;
   struct scan_lists *ports;
-  int range_type = SCAN_TCP_PORT|SCAN_UDP_PORT|SCAN_PROTOCOLS;
+  int range_type = 0;
+
+  if (o.TCPScan())
+    range_type |= SCAN_TCP_PORT;
+  else if (o.UDPScan())
+    range_type |= SCAN_UDP_PORT;
+  else if (o.ipprotscan)
+    range_type |= SCAN_PROTOCOLS;
 
   porttbl = (u8 *) safe_zalloc(65536);
 
@@ -1587,6 +1646,8 @@ printf("%s %s ( %s )\n"
        "  -PS/PA/PU [portlist]: TCP SYN/ACK or UDP discovery to given ports\n"
        "  -PE/PP/PM: ICMP echo, timestamp, and netmask request discovery probes\n"
        "  -n/-R: Never do DNS resolution/Always resolve [default: sometimes]\n"
+       "  --dns-servers <serv1[,serv2],...>: Specify custom DNS servers\n"
+       "  --system-dns: Use OS's DNS resolver\n"
        "SCAN TECHNIQUES:\n"
        "  -sS/sT/sA/sW/sM: TCP SYN/Connect()/ACK/Window/Maimon scans\n"
        "  -sN/sF/sX: TCP Null, FIN, and Xmas scans\n"
@@ -1601,48 +1662,51 @@ printf("%s %s ( %s )\n"
        "  -r: Scan ports consecutively - don't randomize\n"
        "SERVICE/VERSION DETECTION:\n"
        "  -sV: Probe open ports to determine service/version info\n"
-       "  --version_light: Limit to most likely probes for faster identification\n"
-       "  --version_all: Try every single probe for version detection\n"
-       "  --version_trace: Show detailed version scan activity (for debugging)\n"
+       "  --version-intensity <level>: Set from 0 (light) to 9 (try all probes)\n"
+       "  --version-light: Limit to most likely probes (intensity 2)\n"
+       "  --version-all: Try every single probe (intensity 9)\n"
+       "  --version-trace: Show detailed version scan activity (for debugging)\n"
        "OS DETECTION:\n"
        "  -O: Enable OS detection\n"
-       "  --osscan_limit: Limit OS detection to promising targets\n"
-       "  --osscan_guess: Guess OS more aggressively\n"
+       "  --osscan-limit: Limit OS detection to promising targets\n"
+       "  --osscan-guess: Guess OS more aggressively\n"
        "TIMING AND PERFORMANCE:\n"
        "  -T[0-5]: Set timing template (higher is faster)\n"
-       "  --min_hostgroup/max_hostgroup <msec>: Parallel host scan group sizes\n"
-       "  --min_parallelism/max_parallelism <msec>: Probe parallelization\n"
-       "  --min_rtt_timeout/max_rtt_timeout/initial_rtt_timeout <msec>: Specifies\n"
+       "  --min-hostgroup/max-hostgroup <size>: Parallel host scan group sizes\n"
+       "  --min-parallelism/max-parallelism <msec>: Probe parallelization\n"
+       "  --min-rtt-timeout/max-rtt-timeout/initial-rtt-timeout <msec>: Specifies\n"
        "      probe round trip time.\n"
-       "  --host_timeout <msec>: Give up on target after this long\n"
-       "  --scan_delay/--max_scan_delay <msec>: Adjust delay between probes\n"
+       "  --max-retries <tries>: Caps number of port scan probe retransmissions.\n"
+       "  --host-timeout <msec>: Give up on target after this long\n"
+       "  --scan-delay/--max-scan-delay <msec>: Adjust delay between probes\n"
        "FIREWALL/IDS EVASION AND SPOOFING:\n"
        "  -f; --mtu <val>: fragment packets (optionally w/given MTU)\n"
        "  -D <decoy1,decoy2[,ME],...>: Cloak a scan with decoys\n"
        "  -S <IP_Address>: Spoof source address\n"
        "  -e <iface>: Use specified interface\n"
-       "  -g/--source_port <portnum>: Use given port number\n"
-       "  --data_length <num>: Append random data to sent packets\n"
+       "  -g/--source-port <portnum>: Use given port number\n"
+       "  --data-length <num>: Append random data to sent packets\n"
        "  --ttl <val>: Set IP time-to-live field\n"
-       "  --spoof_mac <mac address/prefix/vendor name>: Spoof your MAC address\n"
+       "  --spoof-mac <mac address/prefix/vendor name>: Spoof your MAC address\n"
+       "  --badsum: Send packets with a bogus TCP/UDP checksum\n"
        "OUTPUT:\n"
        "  -oN/-oX/-oS/-oG <file>: Output scan in normal, XML, s|<rIpt kIddi3,\n"
        "     and Grepable format, respectively, to the given filename.\n"
        "  -oA <basename>: Output in the three major formats at once\n"
        "  -v: Increase verbosity level (use twice for more effect)\n"
        "  -d[level]: Set or increase debugging level (Up to 9 is meaningful)\n"
-       "  --packet_trace: Show all packets sent and received\n"
+       "  --packet-trace: Show all packets sent and received\n"
        "  --iflist: Print host interfaces and routes (for debugging)\n"
-       "  --append_output: Append to rather than clobber specified output files\n"
+       "  --append-output: Append to rather than clobber specified output files\n"
        "  --resume <filename>: Resume an aborted scan\n"
        "  --stylesheet <path/URL>: XSL stylesheet to transform XML output to HTML\n"
        "  --webxml: Reference stylesheet from Insecure.Org for more portable XML\n"
-       "  --no_stylesheet: Prevent associating of XSL stylesheet w/XML output\n"
+       "  --no-stylesheet: Prevent associating of XSL stylesheet w/XML output\n"
        "MISC:\n"
        "  -6: Enable IPv6 scanning\n"
        "  -A: Enables OS detection and Version detection\n"
        "  --datadir <dirname>: Specify custom Nmap data file location\n"
-       "  --send_eth/--send_ip: Send using raw ethernet frames or IP packets\n"
+       "  --send-eth/--send-ip: Send using raw ethernet frames or IP packets\n"
        "  --privileged: Assume that the user is fully privileged\n"
        "  -V: Print version number\n"
        "  -h: Print this help summary page.\n"
@@ -1662,13 +1726,13 @@ waits for nmap to finish.  Results are printed to the\n\
 screen (of course you can still use file output commands).\n\
 ! <command>   -- runs shell command given in the foreground\n\
 x             -- Exit Nmap\n\
-f [--spoof <fakeargs>] [--nmap_path <path>] <nmap args>\n\
+f [--spoof <fakeargs>] [--nmap-path <path>] <nmap args>\n\
 -- Executes nmap in the background (results are NOT\n\
 printed to the screen).  You should generally specify a\n\
 file for results (with -oX, -oG, or -oN).  If you specify\n\
 fakeargs with --spoof, Nmap will try to make those\n\
 appear in ps listings.  If you wish to execute a special\n\
-version of Nmap, specify --nmap_path.\n\
+version of Nmap, specify --nmap-path.\n\
 n -h          -- Obtain help with Nmap syntax\n\
 h             -- Prints this help screen.\n\
 Examples:\n\
@@ -1678,23 +1742,8 @@ f --spoof \"/usr/local/bin/pico -z hello.c\" -sS -oN e.log example.com/24\n\n");
 
 char *seqreport(struct seq_info *seq) {
   static char report[512];
-  char tmp[256];
-  char *p;
-  int i;
 
   snprintf(report, sizeof(report), "TCP Sequence Prediction: Class=%s\n                         Difficulty=%d (%s)\n", seqclass2ascii(seq->seqclass), seq->index, seqidx2difficultystr(seq->index));
-  if (o.verbose > 1 || o.debugging ) {
-    p = tmp;
-    strcpy(p, "TCP ISN Seq. Numbers: ");
-    p += 22;
-    for(i=0; i < seq->responses; i++) {
-      if (p - tmp + 20 > ((int) sizeof(tmp)))
-	fatal("0verfl0w Error #234112");
-      p += snprintf(p, 16, "%X ", seq->seqs[i]);
-    }
-    *--p = '\n';
-    strcat(report, tmp);
-  }
   return report;
 }
 
@@ -1889,7 +1938,7 @@ char *grab_next_host_spec(FILE *inputfd, int argc, char **fakeargv) {
     do {
       ip.s_addr = get_random_u32();
     } while (ip_is_reserved(&ip));
-    strcpy(host_spec, inet_ntoa(ip));
+    Strncpy(host_spec, inet_ntoa(ip), sizeof(host_spec));
   } else if (!inputfd) {
     return( (optind < argc)?  fakeargv[optind++] : NULL);
   } else { 
@@ -1926,6 +1975,10 @@ char *scantype2str(stype scantype) {
   case IPPROT_SCAN: return "IPProto Scan"; break;
   case PING_SCAN: return "Ping Scan"; break;
   case PING_SCAN_ARP: return "ARP Ping Scan"; break;
+  case IDLE_SCAN: return "Idle Scan"; break;
+  case BOUNCE_SCAN: return "Bounce Scan"; break;
+  case SERVICE_SCAN: return "Service Scan"; break;
+  case OS_SCAN: return "OS Scan"; break;
   default: assert(0); break;
   }
 
