@@ -98,7 +98,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: nmap.h 2839 2005-09-07 08:26:45Z fyodor $ */
+/* $Id: nmap.h 3014 2005-12-28 08:58:56Z fyodor $ */
 
 #ifndef NMAP_H
 #define NMAP_H
@@ -335,6 +335,10 @@ void *realloc();
 #define INITIAL_RTT_TIMEOUT 1000 /* Allow 1 second initially for packet responses */
 #define HOST_TIMEOUT    0 /* By default allow unlimited time to scan each host */
 
+#ifndef MAX_RETRANSMISSIONS
+#define MAX_RETRANSMISSIONS 10    /* 11 probes to port at maximum */
+#endif
+
 /* If nmap is called with one of the names below, it will start up in interactive mode -- alternatively, you can rename Nmap any of the following names to have it start up interactivey by default.  */
 #define INTERACTIVE_NAMES { "BitchX", "Calendar", "X", "awk", "bash", "bash2", "calendar", "cat", "csh", "elm", "emacs", "ftp", "fvwm", "g++", "gcc", "gimp", "httpd", "irc", "man", "mutt", "nc", "ncftp", "netscape", "perl", "pine", "ping", "sleep", "slirp", "ssh", "sshd", "startx", "tcsh", "telnet", "telnetd", "tia", "top", "vi", "vim", "xdvi", "xemacs", "xterm", "xv" }
 
