@@ -99,14 +99,14 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: nmap_winconfig.h 2871 2005-09-12 10:36:34Z fyodor $ */
+/* $Id: nmap_winconfig.h 3092 2006-01-28 07:57:37Z fyodor $ */
 
 #ifndef NMAP_WINCONFIG_H
 #define NMAP_WINCONFIG_H
 
-#define NMAP_VERSION "3.93"
-#define NMAP_NAME "nmap"
+#define NMAP_VERSION "4.00"
+#define NMAP_NAME "Nmap"
 #define NMAP_URL "http://www.insecure.org/nmap"
 #define NMAP_PLATFORM "i686-pc-windows-windows"
-#define NMAPDATADIR "c:\nmap" /* FIXME: I really need to make this dynamic */
+#define NMAPDATADIR "c:\\nmap" /* FIXME: I really need to make this dynamic */
 #endif /* NMAP_WINCONFIG_H */
