@@ -1,6 +1,6 @@
 
 /***************************************************************************
- * service_scan.h -- Routines used for service fingerprinting to determine *
+ * service_scan.cc -- Routines used for service fingerprinting to determine *
  * what application-level protocol is listening on a given port            *
  * (e.g. snmp, http, ftp, smtp, etc.)                                      *
  *                                                                         *
@@ -98,13 +98,16 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: service_scan.cc 2839 2005-09-07 08:26:45Z fyodor $ */
+/* $Id: service_scan.cc 3024 2005-12-31 00:32:42Z fyodor $ */
 
 
 #include "service_scan.h"
 #include "timing.h"
 #include "NmapOps.h"
 #include "nsock.h"
+
+#include "tty.h"
+
 #if HAVE_OPENSSL
 #include <openssl/ssl.h>
 #endif
@@ -140,7 +143,7 @@ public:
   ~ServiceNFO();
 
   // If a service response to a given probeName, this function adds
-  // the resonse the the fingerprint for that service.  The
+  // the response the the fingerprint for that service.  The
   // fingerprint can be printed when nothing matches the service.  You
   // can obtain the fingerprint (if any) via getServiceFingerprint();
   void addToServiceFingerprint(const char *probeName, const u8 *resp, 
@@ -207,8 +210,8 @@ public:
           
 private:
   // Adds a character to servicefp.  Takes care of word wrapping if
-  // neccessary at the given (wrapat) column.  Chars will only be
-  // written if there is enough space.  Oherwise it exits.
+  // necessary at the given (wrapat) column.  Chars will only be
+  // written if there is enough space.  Otherwise it exits.
   void addServiceChar(char c, int wrapat);
   // Like addServiceChar, but for a whole zero-terminated string
   void addServiceString(char *s, int wrapat);
@@ -395,9 +398,6 @@ void ServiceProbeMatch::InitMatch(const char *matchtext, int lineno) {
   while(isspace(*matchtext)) matchtext++;
     if (*matchtext == '\0' || *matchtext == '\r' || *matchtext == '\n') break;
 
-    if (isSoft)
-      fatal("ServiceProbeMatch::InitMatch: illegal trailing garbage on line %d of nmap-service-probes - note that softmatch lines cannot have a version specifier.", lineno);
-
     modechar = *(matchtext++);
     if (*matchtext == 0 || *matchtext == '\r' || *matchtext == '\n')
       fatal("ServiceProbeMatch::InitMatch: parse error on line %d of nmap-service-probes", lineno);
@@ -974,7 +974,7 @@ void ServiceProbe::setPortVector(vector<u16> *portv, const char *portstr,
   // SERVICE_TUNNEL_SSL.  Otherwise use SERVICE_TUNNEL_NONE.  The line
   // number is requested because this function will bail with an error
   // (giving the line number) if it fails to parse the string.  Ports
-  // are a comma seperated list of prots and ranges
+  // are a comma seperated list of ports and ranges
   // (e.g. 53,80,6000-6010).
 void ServiceProbe::setProbablePorts(enum service_tunnel_type tunnel,
 				    const char *portstr, int lineno) {
@@ -1201,6 +1201,8 @@ int AllProbes::isExcluded(unsigned short port, int proto) {
   unsigned short *p=NULL;
   int count=-1,i;
 
+  if (!excludedports) return 0;
+
   if (proto == IPPROTO_TCP) {
     p = excludedports->tcp_ports;
     count = excludedports->tcp_count;
@@ -1312,8 +1314,8 @@ ServiceNFO::~ServiceNFO() {
 }
 
   // Adds a character to servicefp.  Takes care of word wrapping if
-  // neccessary at the given (wrapat) column.  Chars will only be
-  // written if there is enough space.  Oherwise it exits.
+  // necessary at the given (wrapat) column.  Chars will only be
+  // written if there is enough space.  Otherwise it exits.
 void ServiceNFO::addServiceChar(char c, int wrapat) {
 
   if (servicefpalloc - servicefplen < 6)
@@ -1653,8 +1655,18 @@ ServiceGroup::~ServiceGroup() {
    PORT_OPEN. */
 static void adjustPortStateIfNeccessary(ServiceNFO *svc) {
 
+  char host[128];
+
   if (svc->port->state == PORT_OPENFILTERED) {
-    svc->target->ports.addPort(svc->portno, svc->proto, NULL, PORT_OPEN);
+    svc->port->state = PORT_OPEN;
+
+    if (o.verbose || o.debugging > 1) {
+      svc->target->NameIP(host, sizeof(host));
+
+      log_write(LOG_STDOUT, "Discovered open|filtered port %hu/%s on %s is actually open\n",
+         svc->portno, proto2ascii(svc->proto), host);
+      log_flush(LOG_STDOUT);
+    }
   }
 
   return;
@@ -1800,6 +1812,14 @@ static int scanThroughTunnel(nsock_pool nsp, nsock_iod nsi, ServiceGroup *SG,
 
 /* Prints completion estimates and the like when appropriate */
 static void considerPrintingStats(ServiceGroup *SG) {
+   /* Check for status requests */
+   if (keyWasPressed()) {
+      SG->SPM->printStats(SG->services_finished.size() /
+                          ((double)SG->services_remaining.size() + SG->services_in_progress.size() + 
+                           SG->services_finished.size()), nsock_gettimeofday());
+   }
+
+
   /* Perhaps this should be made more complex, but I suppose it should be
      good enough for now. */
   if (SG->SPM->mayBePrinted(nsock_gettimeofday())) {
@@ -1942,6 +1962,14 @@ void servicescan_write_handler(nsock_pool nsp, nsock_event nse, void *mydata) {
   SG = (ServiceGroup *) nsp_getud(nsp);
   nsi = nse_iod(nse);
 
+  // Check if a status message was requsted
+  if (keyWasPressed()) {
+     SG->SPM->printStats(SG->services_finished.size() /
+                         ((double)SG->services_remaining.size() + SG->services_in_progress.size() + 
+                          SG->services_finished.size()), nsock_gettimeofday());
+  }
+  
+
   if (svc->target->timedOut(nsock_gettimeofday())) {
     end_svcprobe(nsp, PROBESTATE_INCOMPLETE, SG, svc, nsi);
     return;
@@ -2141,6 +2169,22 @@ void servicescan_read_handler(nsock_pool nsp, nsock_event nse, void *mydata) {
   return;
 }
 
+
+// This is used in processResults to determine whether a FP
+// should be printed based on type of match, version intensity, etc.
+
+int shouldWePrintFingerprint(ServiceNFO *svc) {
+  // Never print FP if hardmatched
+  if (svc->probe_state == PROBESTATE_FINISHED_HARDMATCHED)
+    return 0;
+
+  // If we were called with a version_intensity less than
+  // the default, don't bother printing.
+  if (o.version_intensity < 7) return 0;
+
+  return 1;
+}
+
 // This is passed a completed ServiceGroup which contains the scanning results for every service.
 // The function iterates through each finished service and adds the results to Target structure for
 // Nmap to output later.
@@ -2149,7 +2193,7 @@ static void processResults(ServiceGroup *SG) {
 list<ServiceNFO *>::iterator svc;
 
  for(svc = SG->services_finished.begin(); svc != SG->services_finished.end(); svc++) {
-   if ((*svc)->probe_state == PROBESTATE_FINISHED_HARDMATCHED) {
+   if ((*svc)->probe_state != PROBESTATE_FINISHED_NOMATCH) {
      (*svc)->port->setServiceProbeResults((*svc)->probe_state, 
 					  (*svc)->probe_matched,
 					  (*svc)->tunnel,
@@ -2159,16 +2203,8 @@ list<ServiceNFO *>::iterator svc;
 					  *(*svc)->hostname_matched? (*svc)->hostname_matched : NULL, 
 					  *(*svc)->ostype_matched? (*svc)->ostype_matched : NULL, 
 					  *(*svc)->devicetype_matched? (*svc)->devicetype_matched : NULL, 
-					  NULL);
-
-   } else if ((*svc)->probe_state == PROBESTATE_FINISHED_SOFTMATCHED) {
-    (*svc)->port->setServiceProbeResults((*svc)->probe_state, 
-					  (*svc)->probe_matched,
-					 (*svc)->tunnel,
-					  NULL, NULL, NULL, NULL, NULL, NULL, 
-					 (*svc)->getServiceFingerprint(NULL));
-
-   }  else if ((*svc)->probe_state == PROBESTATE_FINISHED_NOMATCH) {
+					  shouldWePrintFingerprint(*svc) ? (*svc)->getServiceFingerprint(NULL) : NULL);
+   }  else {
      if ((*svc)->getServiceFingerprint(NULL))
        (*svc)->port->setServiceProbeResults((*svc)->probe_state, NULL,
 					    (*svc)->tunnel, NULL, NULL, NULL, NULL, NULL, NULL,
