@@ -98,7 +98,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: targets.cc 2950 2005-11-28 01:34:08Z fyodor $ */
+/* $Id: targets.cc 3039 2006-01-11 01:32:04Z fyodor $ */
 
 
 #include "targets.h"
@@ -108,6 +108,8 @@
 #include "TargetGroup.h"
 #include "Target.h"
 #include "scan_engine.h"
+#include "nmap_dns.h"
+#include "tty.h"
 
 using namespace std;
 extern NmapOps o;
@@ -433,6 +435,8 @@ if (hs->randomize) {
    else
      massping(hs->hostbatch, hs->current_batch_sz, ports, *pingtype);
  
+ if (!o.noresolve) nmap_mass_rdns(hs->hostbatch, hs->current_batch_sz);
+ 
  return hs->hostbatch[hs->next_batch_no++];
 }
 
@@ -980,6 +984,9 @@ pingpkt.id = id;
 pingpkt.seq = seq;
 pingpkt.checksum = 0;
 pingpkt.checksum = in_cksum((unsigned short *)ping, icmplen);
+if ( o.badsum )
+  pingpkt.checksum--;
+
 
 /* Now for our sock */
 if (ptech.icmpscan) {
@@ -1203,6 +1210,8 @@ int get_ping_results(int sd, pcap_t *pd, Target *hostbatch[], int pingtype,
   newportstate = PORT_UNKNOWN;
 
   while(pt->block_unaccounted > 0 && !timeout) {
+    keyWasPressed(); // Check for status message printing
+
     tmpto = myto;
 
     if (pd) {
