@@ -98,7 +98,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: tcpip.cc 2863 2005-09-11 10:55:03Z fyodor $ */
+/* $Id: tcpip.cc 3090 2006-01-26 09:50:57Z fyodor $ */
 
 #include <dnet.h>
 #include "tcpip.h"
@@ -449,7 +449,7 @@ const char *ippackethdrinfo(const u8 *packet, u32 len) {
     tcp = (struct tcphdr *)  (packet + ip->ip_hl * 4);
     if (frag_off > 8 || len < (u32) ip->ip_hl * 4 + 8) 
       snprintf(protoinfo, sizeof(protoinfo), "TCP %s:?? > %s:?? ?? %s (incomplete)", srchost, dsthost, ipinfo);
-    else if (frag_off == 8 && len >= (u32) ip->ip_hl * 4 + 8) {// we can get TCP flags nad ACKn
+    else if (frag_off == 8 && len >= (u32) ip->ip_hl * 4 + 8) {// we can get TCP flags and ACKn
       tcp = (struct tcphdr *)((u8 *) tcp - frag_off); // ugly?
       p = tflags;
       /* These are basically in tcpdump order */
@@ -875,8 +875,11 @@ else tcp->th_win = htons(1024 * (myttl % 4 + 1)); /* Who cares */
 tcp->th_sum = in_cksum((unsigned short *)pseudo, sizeof(struct tcphdr) + 
 		       optlen + sizeof(struct pseudo_header) + datalen);
 #endif
-/* Now for the ip header */
 
+if ( o.badsum )
+  --tcp->th_sum;
+
+/* Now for the ip header */
 memset(packet, 0, sizeof(struct ip)); 
 ip->ip_v = 4;
 ip->ip_hl = 5;
@@ -984,7 +987,7 @@ int send_ip_packet(int sd, struct eth_nfo *eth, u8 *packet, unsigned int packetl
      must deal with it here rather than when building the packet,
      because they should be in NBO when I'm sending over raw
      ethernet */
-#if FREEBSD || BSDI || NETBSD || DEC
+#if FREEBSD || BSDI || NETBSD || DEC || MACOSX
   ip->ip_len = ntohs(ip->ip_len);
   ip->ip_off = ntohs(ip->ip_off);
 #endif
@@ -1093,6 +1096,9 @@ pingpkt.seq = seq;
 pingpkt.checksum = 0;
 pingpkt.checksum = in_cksum((unsigned short *)ping, icmplen);
 
+if ( o.badsum )
+  --pingpkt.checksum;
+
 return build_ip_raw(source, victim, o.ttl, IPPROTO_ICMP, get_random_u16(),
 		    ping, icmplen, packetlen);
 }
@@ -1281,7 +1287,7 @@ u8 *build_udp_raw(struct in_addr *source, const struct in_addr *victim,
   if (data)
     memcpy(packet + sizeof(struct ip) + sizeof(udphdr_bsd), data, datalen);
   
-  /* Now the psuedo header for checksuming */
+  /* Now the pseudo header for checksuming */
   pseudo->source.s_addr = source->s_addr;
   pseudo->dest.s_addr = victim->s_addr;
   pseudo->proto = IPPROTO_UDP;
@@ -1294,6 +1300,9 @@ u8 *build_udp_raw(struct in_addr *source, const struct in_addr *victim,
   udp->uh_sum = in_cksum((unsigned short *)pseudo, 20 /* pseudo + UDP headers */ + datalen);
 #endif
   
+  if ( o.badsum )
+    --udp->uh_sum;
+  
   /* Goodbye, pseudo header! */
   memset(pseudo, 0, sizeof(*pseudo));
   
@@ -1379,10 +1388,10 @@ ip->ip_id = htons(ipid);
 ip->ip_ttl = myttl;
 ip->ip_p = proto;
 ip->ip_src.s_addr = source->s_addr;
-#ifdef WIN32
+// #ifdef WIN32
 // TODO: Should this be removed? I'm not sure why this is here -- Fyodor
-if(source->s_addr == victim->s_addr) ip->ip_src.s_addr++;
-#endif
+// if(source->s_addr == victim->s_addr) ip->ip_src.s_addr++;
+// #endif
 ip->ip_dst.s_addr = victim->s_addr;
 #if HAVE_IP_IP_SUM
 ip->ip_sum = in_cksum((unsigned short *)ip, sizeof(struct ip));
@@ -1620,7 +1629,7 @@ if (timedout) {
  return alignedbuf;
 }
  
-// Returns whether the packet receive time value obtaned from libpcap
+// Returns whether the packet receive time value obtained from libpcap
 // (and thus by readip_pcap()) should be considered valid.  When
 // invalid (Windows and Amiga), readip_pcap returns the time you called it.
 bool pcap_recv_timeval_valid() {
@@ -1633,7 +1642,7 @@ bool pcap_recv_timeval_valid() {
 
 
 /* A trivial functon that maintains a cache of IP to MAC Address
-   entries.  If the command is ARPCACHE_GEt, this func looks for the
+   entries.  If the command is ARPCACHE_GET, this func looks for the
    IPv4 address in ss and fills in the 'mac' parameter and returns
    true if it is found.  Otherwise (not found), the function returns
    false.  If the command is ARPCACHE_SET, the function adds an entry
@@ -1932,7 +1941,7 @@ bool setTargetNextHopMAC(Target *target) {
   if (target->ifType() != devt_ethernet)
     return false; /* Duh. */
 
-  /* First check if we alread have it, duh. */
+  /* First check if we already have it, duh. */
   if (target->NextHopMACAddress())
     return true;
 
@@ -2134,14 +2143,14 @@ int collect_dnet_interfaces(const struct intf_entry *entry, void *arg) {
    Strncpy(dcrn->ifaces[numifaces].devfullname, entry->intf_name, sizeof(dcrn->ifaces[numifaces].devfullname));
 
    /* Interface type */
-   if (entry->intf_type & INTF_TYPE_ETH) {
+   if (entry->intf_type == INTF_TYPE_ETH) {
 	   dcrn->ifaces[numifaces].device_type = devt_ethernet;
 	   /* Collect the MAC address since this is ethernet */
 	   memcpy(dcrn->ifaces[numifaces].mac, &entry->intf_link_addr.addr_eth.data, 6);
    }
-   else if (entry->intf_type & INTF_TYPE_LOOPBACK)
+   else if (entry->intf_type == INTF_TYPE_LOOPBACK)
 	   dcrn->ifaces[numifaces].device_type = devt_loopback;
-   else if (entry->intf_type & INTF_TYPE_TUN)
+   else if (entry->intf_type == INTF_TYPE_TUN)
 	   dcrn->ifaces[numifaces].device_type = devt_p2p;
    else dcrn->ifaces[numifaces].device_type = devt_other;
    
@@ -2226,8 +2235,8 @@ int sd;
     printf("Size of struct ifreq: %d\n", sizeof(struct ifreq));
 #endif
 
-    for(; ifr && *((u8 *)ifr) && ((u8 *)ifr) < buf + ifc.ifc_len; 
-	((*(char **)&ifr) += len )) {
+    for(; ifr && ifr->ifr_name[0] && ((u8 *)ifr) < buf + ifc.ifc_len;
+	ifr = (struct ifreq *)(((char *)ifr) + len)) {
 #if TCPIP_DEBUGGING
       printf("ifr_name size = %d\n", sizeof(ifr->ifr_name));
       printf("ifr = %X\n",(unsigned)(*(char **)&ifr));
@@ -2296,7 +2305,7 @@ int sd;
 	eth_addr_t ethaddr;
 
 	if (!ethsd) 
-	  fatal("%s: Failed to open ethernet interface (%s)", __FUNCTION__,
+	  fatal("%s: Failed to open ethernet interface (%s). A possible cause on BSD operating systems is running out of BPF devices (see http://seclists.org/lists/nmap-dev/2006/Jan-Mar/0014.html).", __FUNCTION__,
 		mydevs[numifaces].devname);
 	if (eth_get(ethsd, &ethaddr) != 0) 
 	  fatal("%s: Failed to obtain MAC address for ethernet interface (%s)",
@@ -2327,7 +2336,48 @@ int sd;
   if (howmany) *howmany = numifaces;
   return mydevs;
 }
- 
+
+/* Looks for an interface assigned to the given IP (ss), and returns
+   the interface_info for the first one found.  If non found, returns NULL */
+struct interface_info *getInterfaceByIP(struct sockaddr_storage *ss) {
+  struct sockaddr_in *sin = (struct sockaddr_in *) ss;
+  struct sockaddr_in *ifsin;
+  struct interface_info *ifaces;
+  int numifaces = 0;
+  int ifnum;
+
+  if (sin->sin_family != AF_INET)
+    fatal("%s called with non-IPv4 address", __FUNCTION__);
+
+  ifaces = getinterfaces(&numifaces);
+
+  for(ifnum=0; ifnum < numifaces; ifnum++) {
+    ifsin = (struct sockaddr_in *) &ifaces[ifnum].addr;
+    if (ifsin->sin_family != AF_INET) continue;
+    if (sin->sin_addr.s_addr == ifsin->sin_addr.s_addr)
+      return &ifaces[ifnum];
+  }
+  return NULL;
+}
+
+/* Looks for an interface with the given name (iname), and returns the
+   corresponding interface_info if found.  Will accept a match of
+   devname or devfullname.  Returns NULL if none found */
+struct interface_info *getInterfaceByName(char *iname) {
+  struct interface_info *ifaces;
+  int numifaces = 0;
+  int ifnum;
+
+  ifaces = getinterfaces(&numifaces);
+
+  for(ifnum=0; ifnum < numifaces; ifnum++) {
+    if (strcmp(ifaces[ifnum].devfullname, iname) == 0 ||
+	strcmp(ifaces[ifnum].devname, iname) == 0)
+      return &ifaces[ifnum];
+  }
+
+  return NULL;
+}
 
 
 /* A trivial function used with qsort to sort the routes by netmask */
@@ -2488,11 +2538,15 @@ struct sys_route *getsysroutes(int *howmany) {
    source address and interface necessary to route to this address.
    If no route is found, false is returned and rnfo is undefined.  If
    a route is found, true is returned and rnfo is filled in with all
-   of the routing details */
+   of the routing details.  This function takes into account -S and -e
+   options set by user (o.spoofsource, o.device) */
 bool route_dst(const struct sockaddr_storage *const dst, struct route_nfo *rnfo) {
   struct interface_info *ifaces;
+  struct interface_info *iface = NULL;
   int numifaces = 0;
   struct sys_route *routes;
+  struct sockaddr_storage spoofss;
+  size_t spoofsslen;
   int numroutes = 0;
   int ifnum;
   int i;
@@ -2504,6 +2558,56 @@ bool route_dst(const struct sockaddr_storage *const dst, struct route_nfo *rnfo)
   if (dstsin->sin_family != AF_INET)
     fatal("Sorry -- route_dst currently only supports IPv4");
 
+  /* First let us deal with the case where a user requested a specific spoofed IP/dev */
+  if (o.spoofsource || *o.device) {
+    if (o.spoofsource) {
+      o.SourceSockAddr(&spoofss, &spoofsslen);
+	if (!*o.device) {
+	  /* Look up the device corresponding to src IP, if any ... */
+	  iface = getInterfaceByIP(&spoofss);
+	}
+    }
+
+    if (*o.device) {
+      iface = getInterfaceByName(o.device);
+      if (!iface) 
+	fatal("Could not find interface %s which was specified by -e", o.device);
+    }
+
+    if (iface) {
+      /* Is it directly connected? */
+      mask = htonl((unsigned long) (0-1) << (32 - iface->netmask_bits));
+      ifsin = (struct sockaddr_in *) &(iface->addr);
+      if ((ifsin->sin_addr.s_addr & mask) == (dstsin->sin_addr.s_addr & mask))
+	rnfo->direct_connect = 1;
+      else {
+	rnfo->direct_connect = 0;
+	/* must find the next hop by checking route table ... */
+	routes = getsysroutes(&numroutes);
+	/* Now we simply go through the list and take the first match */
+	for(i=0; i < numroutes; i++) {
+	  if (strcmp(routes[i].device->devname, iface->devname) == 0 && 
+	      (routes[i].dest & routes[i].netmask) == 
+	      (dstsin->sin_addr.s_addr & routes[i].netmask)) {
+	    /* Yay, found a matching route. */
+	    ifsin = (struct sockaddr_in *) &rnfo->nexthop;
+	    ifsin->sin_family = AF_INET;
+	    ifsin->sin_addr.s_addr = routes[i].gw.s_addr;
+	  }
+	}
+      }
+      memcpy(&rnfo->ii, iface, sizeof(rnfo->ii));
+      if (o.spoofsource)
+	memcpy(&rnfo->srcaddr, &spoofss, sizeof(rnfo->srcaddr));
+      else
+	memcpy(&rnfo->srcaddr, &(iface->addr), sizeof(rnfo->srcaddr));
+      return true;
+    }
+    /* Control will get here if -S was specified to a non-interface
+       IP, but no interface was specified with -e.  We will try to
+       determine the proper interface in that case */
+  }
+
   ifaces = getinterfaces(&numifaces);
   /* I suppose that I'll first determine whether it is a direct connect instance */
   for(ifnum=0; ifnum < numifaces; ifnum++) {
@@ -2520,7 +2624,10 @@ bool route_dst(const struct sockaddr_storage *const dst, struct route_nfo *rnfo)
 	rnfo->direct_connect = true;
 	memcpy(&rnfo->ii, &ifaces[i], sizeof(rnfo->ii));
 	/* But the source address we want to use is the target addy */
-	memcpy(&rnfo->srcaddr, &ifaces[ifnum].addr, sizeof(rnfo->srcaddr));
+	if (o.spoofsource)
+	  memcpy(&rnfo->srcaddr, &spoofss, sizeof(rnfo->srcaddr));
+	else
+	  memcpy(&rnfo->srcaddr, &ifaces[ifnum].addr, sizeof(rnfo->srcaddr));
 	return true;
       }
       /* Hmmm ... no localhost -- I guess I'll just try using the device 
@@ -2530,7 +2637,10 @@ bool route_dst(const struct sockaddr_storage *const dst, struct route_nfo *rnfo)
     if ((ifsin->sin_addr.s_addr & mask) == (dstsin->sin_addr.s_addr & mask)) {
       rnfo->direct_connect = 1;
       memcpy(&rnfo->ii, &ifaces[ifnum], sizeof(rnfo->ii));
-      memcpy(&rnfo->srcaddr, &ifaces[ifnum].addr, sizeof(rnfo->srcaddr));
+      if (o.spoofsource)
+	memcpy(&rnfo->srcaddr, &spoofss, sizeof(rnfo->srcaddr));
+      else
+	memcpy(&rnfo->srcaddr, &ifaces[ifnum].addr, sizeof(rnfo->srcaddr));
       return true;
     }
   }
@@ -2545,7 +2655,10 @@ bool route_dst(const struct sockaddr_storage *const dst, struct route_nfo *rnfo)
 	(dstsin->sin_addr.s_addr & routes[i].netmask)) {
       /* Yay, found a matching route. */
       memcpy(&rnfo->ii, routes[i].device, sizeof(rnfo->ii));
-      memcpy(&rnfo->srcaddr, &routes[i].device->addr, sizeof(rnfo->srcaddr));
+      if (o.spoofsource)
+	memcpy(&rnfo->srcaddr, &spoofss, sizeof(rnfo->srcaddr));
+      else
+	memcpy(&rnfo->srcaddr, &routes[i].device->addr, sizeof(rnfo->srcaddr));
       ifsin = (struct sockaddr_in *) &rnfo->nexthop;
       ifsin->sin_family = AF_INET;
       ifsin->sin_addr.s_addr = routes[i].gw.s_addr;
@@ -2635,7 +2748,7 @@ void broadcast_socket(int sd) {
   }
 }
 
-/* Do a receive (recv()) on a socket and stick the results (upt to
+/* Do a receive (recv()) on a socket and stick the results (up to
    len) into buf .  Give up after 'seconds'.  Returns the number of
    bytes read (or -1 in the case of an error.  It only does one recv
    (it will not keep going until len bytes are read).  If timedout is
